#
# Vetsoft Backup Script
# V1.1; Eric Walter, 28.08.2015
#
# Parameter: 
#   $1: Anzahl alter backups, die bereitgestellt werden
#    oder
#   $1=LT: LongTerm Backup; es wird alles n ein separates Verzeichnis gesichert
#
# V1.0: initial release
# V1.1: zusaetzl. Verz. via Parameter sichern "USERDIRS"
#


SCRIPT=$(readlink -f $0)
SCRIPTHOME=`dirname $SCRIPT`
VSHOME=`echo $SCRIPTHOME | awk '{split($0,a,"admin"); print a[1]}'`
VSHOME=${VSHOME::-1}
VSDATAHOME=${VSHOME}data

BCKROOT=/backup
DATUM=`date '+%Y%m%d-%H:%M:%S'`
BCKDIR=$BCKROOT/data/$DATUM
DUMPFILE=$BCKDIR/vsDB.dump
VSTARFILE=$BCKDIR/vs.tar.bz2
VSDATATARFILE=$BCKDIR/vsdata.tar.bz2
USERDIRS="/home"

VSDB=vsdb

#erzeuge neues Backup-Dir 
mkdir -p $BCKDIR

# Test Parameter 1 numerisch?
if [ "$1" -eq "$1" ] 2>/dev/null
then
  # wenn $1 nummerisch, dann kein LT-Backup
  KEEPDIRS=$1
  ALLBCKDIRS=`ls -d $BCKROOT/data/2* | sort -r`
else
  if [ "$1" == "LT" ] ; then
    BCKDIR=$BCKROOT/LT-data/$DATUM
    DUMPFILE=$BCKDIR/vsDB.dump
    VSTARFILE=$BCKDIR/vs.tar.bz2
    VSDATATARFILE=$BCKDIR/vsdata.tar.bz2
    ALLBCKDIRS=""
  else
    echo "ERROR: first paramter must be an integer or string LT"
    echo "`basename $0` <anzahl bereitgestellter Backups> | LT"
    exit 1
  fi
fi

#loeschen alter Backups
ct=1
for d in $ALLBCKDIRS ; do
  if [ $ct -gt "$KEEPDIRS" ] ; then
    rm -rf $d
  fi
  let ct=ct+1
done

# schreibrechte fuer user postgres wg. dump
chown postgres $BCKDIR

# DB Dump
sudo -i -u postgres pg_dump -Fc $VSDB > $DUMPFILE
bzip2 $DUMPFILE

# Website & Vetsoft-Data in separate TAR-Archive
tar cjf $VSTARFILE -C $VSHOME  .

if [ "$1" == "LT" ] ; then
  tar cjf $VSDATATARFILE -C $VSDATAHOME .
else
  rsync -rtv $VSDATAHOME $BCKROOT/data/vsdata > /dev/null 2>&1
fi

for dir in $USERDIRS ; do
  TARFNAME=`basename $dir`
  tar cjf $BCKDIR/$TARFNAME.tar.bz2 -C $dir .
done 
