-- Description:	SQL-Skript zum Aktualisieren der Vetsoft Office DB von Version 1.81 auf 1.82
-- Author:		Dirk Walter, Klaus Munsteiner
-- Create Date:	2015-08-10


----------------------------------
-- Teil 1 - Strukturaenderungen --
----------------------------------

-- Tabelle vs_behandlung
-- Column: rechnung_neu



-- Zusaetzliche Felder fuer STORNO Anfang --

-- Tabelle vs_rechnung
-- Column: storno_datum
-- Column: storno_aktiv

-- ALTER TABLE vs_rechnung ADD COLUMN storno_datum;
-- ALTER TABLE vs_rechnung ADD COLUMN storno_aktiv;


ALTER TABLE vs_rechnung ADD COLUMN storno_datum date;
COMMENT ON COLUMN vs_rechnung.storno_datum IS '2015-08-10 KM: Neu fuer STORNO Datum';

ALTER TABLE vs_rechnung ADD COLUMN storno_aktiv character varying(1) DEFAULT 0;
COMMENT ON COLUMN vs_rechnung.storno_aktiv IS '2015-08-10 KM: Neu fuer STORNO aktiv';

-- View: vs_v_rechnung

DROP VIEW vs_v_rechnung;

CREATE VIEW vs_v_rechnung AS
  SELECT r.rechnung_id,
     r.besitzer_id,
     r.rechnungdatum,
     r.ausgabedatum,
     r.ausgabeart,
     r.memo,
     r.zahlfrist,
     r.rechnungnummer,
     r.mahnung_1_datum,
     r.mahnung_1_gebuehr,
     r.mahnung_2_datum,
     r.mahnung_2_gebuehr,
     r.mahnung_3_datum,
     r.mahnung_3_gebuehr,
     r.zusatztext,
     r.storno_datum,
     r.storno_aktiv,
     b.anrede,
     b.briefanrede,
     b.titel,
     b.vorname,
     b.name,
     b.strasse,
     b.plz,
     b.wohnort,
     b.mail,
     ( SELECT sum(vs_behandlung.brutto) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS brutto,
     ( SELECT sum(vs_behandlung.netto) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS netto,
     ( SELECT sum(vs_behandlung.betragmwst) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS betragmwst,
     ( SELECT sum(vs_behandlung.brutto_a) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS brutto_a,
     ( SELECT sum(vs_behandlung.brutto_b) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS brutto_b,
     ( SELECT sum(vs_behandlung.brutto_abgabemed) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS  
brutto_abgabemed,
     ( SELECT sum(vs_behandlung.bezahlt) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS bezahlt,
     ( SELECT sum(vs_behandlung.offen) AS sum
            FROM vs_behandlung
           WHERE vs_behandlung.rechnung_id = r.rechnung_id) AS offen
    FROM vs_rechnung r,
     vs_besitzer b
   WHERE r.besitzer_id = b.besitzer_id;

-- Zusaetzliche Felder fuer STORNO Ende --



-- Neue Tabelle: vs_db_version

-- Column: db_version_id
-- Column: db_version
-- Column: db_version_aktiv
-- Column: db_version_bemerkung


CREATE TABLE vs_db_version ()
WITH (OIDS=FALSE);
ALTER TABLE vs_db_version OWNER TO postgres;

ALTER TABLE vs_db_version ADD COLUMN db_version_id serial PRIMARY KEY;
COMMENT ON COLUMN vs_db_version.db_version_id IS '2015-08-23 KM: Datensatz-Nummer';

ALTER TABLE vs_db_version ADD COLUMN db_version character varying(10);
COMMENT ON COLUMN vs_db_version.db_version IS '2015-08-23 KM: Vetsoft Office DB-Version';

ALTER TABLE vs_db_version ADD COLUMN db_version_aktiv character varying(1) DEFAULT 0;
COMMENT ON COLUMN vs_db_version.db_version_aktiv IS '2015-08-23 KM: Aktive Version = "1", Default = "0"';

ALTER TABLE vs_db_version ADD COLUMN db_version_bemerkung character varying(255);
COMMENT ON COLUMN vs_db_version.db_version_bemerkung IS '2015-08-23 KM: Bemerkungen zur DB-Version';




---------------------------------------------------------------
-- Teil 2 - Datenaenderungen
---------------------------------------------------------------

-- vs_rechnung
-- Spalten DEFAULT-Werte setzen
-------------------

-- storno_datum darf nicht leer sein, erst mal '0001-01-01' rein schreiben.
UPDATE vs_rechnung SET storno_datum = '0001-01-01' WHERE storno_datum IS NULL;

-- storno_aktiv darf nicht leer sein, DEFAULT = 0
UPDATE vs_rechnung SET storno_aktiv = 0 WHERE storno_aktiv IS NULL;



-- vs_db_version 
-- Datensaetze einfuegen
------------------------

INSERT INTO vs_db_version (db_version, db_version_aktiv, db_version_bemerkung)
   VALUES('1.74', '0', 'Datenbankupgrade fuer Version 1.74');

INSERT INTO vs_db_version (db_version, db_version_aktiv, db_version_bemerkung)
   VALUES('1.81', '0', 'Datenbankupgrade fuer neue Einzelrechnung erstellen');

INSERT INTO vs_db_version (db_version, db_version_aktiv, db_version_bemerkung)
   VALUES('1.82', '1', 'Datenbankupgrade fuer Versionskontrolle und STORNO');






----------------------------------------------------------------
-- Teil 3 - FIX SEQUENCE: ZAEHLER UEBERPRUFEN UND KORRIGIEREN --
-- Create Date:	2015-08-28
----------------------------------------------------------------


SELECT SETVAL('public.vs_anrede_anrede_id_seq', COALESCE(MAX(anrede_id), 1) ) FROM public.vs_anrede;
SELECT SETVAL('public.vs_behandlung_behandlung_id_seq', COALESCE(MAX(behandlung_id), 1) ) FROM public.vs_behandlung;
SELECT SETVAL('public.vs_behdoc_behdoc_id_seq', COALESCE(MAX(behdoc_id), 1) ) FROM public.vs_behdoc;
SELECT SETVAL('public.vs_behlabor_behlabor_id_seq', COALESCE(MAX(behlabor_id), 1) ) FROM public.vs_behlabor;
SELECT SETVAL('public.vs_behposition_behposition_id_seq', COALESCE(MAX(behposition_id), 1) ) FROM public.vs_behposition;
SELECT SETVAL('public.vs_bes_event_bes_event_id_seq', COALESCE(MAX(bes_event_id), 1) ) FROM public.vs_bes_event;
SELECT SETVAL('public.vs_bes_event_typ_bes_event_typ_id_seq', COALESCE(MAX(bes_event_typ_id), 1) ) FROM public.vs_bes_event_typ;
SELECT SETVAL('public.vs_besitzer_besitzer_id_seq', COALESCE(MAX(besitzer_id), 1) ) FROM public.vs_besitzer;
SELECT SETVAL('public.vs_bestellung_bestellung_id_seq', COALESCE(MAX(bestellung_id), 1) ) FROM public.vs_bestellung;
SELECT SETVAL('public.vs_bezahlt_bezahlt_id_seq', COALESCE(MAX(bezahlt_id), 1) ) FROM public.vs_bezahlt;
SELECT SETVAL('public.vs_briefanrede_briefanrede_id_seq', COALESCE(MAX(briefanrede_id), 1) ) FROM public.vs_briefanrede;
SELECT SETVAL('public.vs_etikette_etikette_id_seq', COALESCE(MAX(etikette_id), 1) ) FROM public.vs_etikette;
SELECT SETVAL('public.vs_fremdleistung_fremdleistung_id_seq', COALESCE(MAX(fremdleistung_id), 1) ) FROM public.vs_fremdleistung;
SELECT SETVAL('public.vs_futter_futter_id_seq', COALESCE(MAX(futter_id), 1) ) FROM public.vs_futter;
SELECT SETVAL('public.vs_futter_gruppe_futter_gruppe_id_seq', COALESCE(MAX(futter_gruppe_id), 1) ) FROM public.vs_futter_gruppe;
SELECT SETVAL('public.vs_gruppe1_gruppe1_id_seq', COALESCE(MAX(gruppe1_id), 1) ) FROM public.vs_gruppe1;
SELECT SETVAL('public.vs_gruppe2_gruppe2_id_seq', COALESCE(MAX(gruppe2_id), 1) ) FROM public.vs_gruppe2;
SELECT SETVAL('public.vs_gruppe3_gruppe3_id_seq', COALESCE(MAX(gruppe3_id), 1) ) FROM public.vs_gruppe3;
SELECT SETVAL('public.vs_gruppe4_gruppe4_id_seq', COALESCE(MAX(gruppe4_id), 1) ) FROM public.vs_gruppe4;
SELECT SETVAL('public.vs_gruppe5_gruppe5_id_seq', COALESCE(MAX(gruppe5_id), 1) ) FROM public.vs_gruppe5;
SELECT SETVAL('public.vs_impfung_impfung_id_seq', COALESCE(MAX(impfung_id), 1) ) FROM public.vs_impfung;
SELECT SETVAL('public.vs_impfung_zugang_impfung_zugang_id_seq', COALESCE(MAX(impfung_zugang_id), 1) ) FROM public.vs_impfung_zugang;
SELECT SETVAL('public.vs_kasse_kasse_id_seq', COALESCE(MAX(kasse_id), 1) ) FROM public.vs_kasse;
SELECT SETVAL('public.vs_labor_einheit_labor_einheit_id_seq', COALESCE(MAX(labor_einheit_id), 1) ) FROM public.vs_labor_einheit;
SELECT SETVAL('public.vs_labor_labor_id_seq', COALESCE(MAX(labor_id), 1) ) FROM public.vs_labor;
SELECT SETVAL('public.vs_laborus_laborus_id_seq', COALESCE(MAX(laborus_id), 1) ) FROM public.vs_laborus;
SELECT SETVAL('public.vs_laborwert_laborwert_id_seq', COALESCE(MAX(laborwert_id), 1) ) FROM public.vs_laborwert;
SELECT SETVAL('public.vs_leistung_leistung_id_seq', COALESCE(MAX(leistung_id), 1) ) FROM public.vs_leistung;
SELECT SETVAL('public.vs_lieferant_lieferant_id_seq', COALESCE(MAX(lieferant_id), 1) ) FROM public.vs_lieferant;
SELECT SETVAL('public.vs_med_gruppe_med_gruppe_id_seq', COALESCE(MAX(med_gruppe_id), 1) ) FROM public.vs_med_gruppe;
SELECT SETVAL('public.vs_med_typ_med_typ_id_seq', COALESCE(MAX(med_typ_id), 1) ) FROM public.vs_med_typ;
SELECT SETVAL('public.vs_med_zugang_med_zugang_id_seq', COALESCE(MAX(med_zugang_id), 1) ) FROM public.vs_med_zugang;
SELECT SETVAL('public.vs_medikament_medikament_id_seq', COALESCE(MAX(medikament_id), 1) ) FROM public.vs_medikament;
SELECT SETVAL('public.vs_mitarbeiter_mitarbeiter_id_seq', COALESCE(MAX(mitarbeiter_id), 1) ) FROM public.vs_mitarbeiter;
SELECT SETVAL('public.vs_mustertext_mustertext_id_seq', COALESCE(MAX(mustertext_id), 1) ) FROM public.vs_mustertext;
SELECT SETVAL('public.vs_patient_patient_id_seq', COALESCE(MAX(patient_id), 1) ) FROM public.vs_patient;
SELECT SETVAL('public.vs_patimpfung_patimpfung_id_seq', COALESCE(MAX(patimpfung_id), 1) ) FROM public.vs_patimpfung;
SELECT SETVAL('public.vs_rasse_rasse_id_seq', COALESCE(MAX(rasse_id), 1) ) FROM public.vs_rasse;
SELECT SETVAL('public.vs_rech_bez_rech_bez_id_seq', COALESCE(MAX(rech_bez_id), 1) ) FROM public.vs_rech_bez;
SELECT SETVAL('public.vs_rechnung_rechnung_id_seq', COALESCE(MAX(rechnung_id), 1) ) FROM public.vs_rechnung;
SELECT SETVAL('public.vs_roe_worklist_roe_worklist_id_seq', COALESCE(MAX(roe_worklist_id), 1) ) FROM public.vs_roe_worklist;
SELECT SETVAL('public.vs_roentgen_roentgen_id_seq', COALESCE(MAX(roentgen_id), 1) ) FROM public.vs_roentgen;
SELECT SETVAL('public.vs_serienbrief_serienbrief_id_seq', COALESCE(MAX(serienbrief_id), 1) ) FROM public.vs_serienbrief;
SELECT SETVAL('public.vs_standard_standard_id_seq', COALESCE(MAX(standard_id), 1) ) FROM public.vs_standard;
SELECT SETVAL('public.vs_stapos_stapos_id_seq', COALESCE(MAX(stapos_id), 1) ) FROM public.vs_stapos;
SELECT SETVAL('public.vs_status_status_id_seq', COALESCE(MAX(status_id), 1) ) FROM public.vs_status;
SELECT SETVAL('public.vs_textbaustein_textbaustein_id_seq', COALESCE(MAX(textbaustein_id), 1) ) FROM public.vs_textbaustein;
SELECT SETVAL('public.vs_tierart_tierart_id_seq', COALESCE(MAX(tierart_id), 1) ) FROM public.vs_tierart;
SELECT SETVAL('public.vs_titel_titel_id_seq', COALESCE(MAX(titel_id), 1) ) FROM public.vs_titel;
SELECT SETVAL('public.vs_user_user_id_seq', COALESCE(MAX(user_id), 1) ) FROM public.vs_user;
SELECT SETVAL('public.vs_verbrauchsmaterial_gruppe_verbrauchsmaterial_gruppe_id_seq', COALESCE(MAX(verbrauchsmaterial_gruppe_id), 1) ) FROM public.vs_verbrauchsmaterial_gruppe;
SELECT SETVAL('public.vs_verbrauchsmaterial_verbrauchsmaterial_id_seq', COALESCE(MAX(verbrauchsmaterial_id), 1) ) FROM public.vs_verbrauchsmaterial;
SELECT SETVAL('public.vs_waren_gruppe_waren_gruppe_id_seq', COALESCE(MAX(waren_gruppe_id), 1) ) FROM public.vs_waren_gruppe;
SELECT SETVAL('public.vs_waren_waren_id_seq', COALESCE(MAX(waren_id), 1) ) FROM public.vs_waren;
SELECT SETVAL('public.vs_wartezimmer_wartezimmer_id_seq', COALESCE(MAX(wartezimmer_id), 1) ) FROM public.vs_wartezimmer;
SELECT SETVAL('public.vs_wissen_wissen_id_seq', COALESCE(MAX(wissen_id), 1) ) FROM public.vs_wissen;
SELECT SETVAL('public.vs_wwwlink_wwwlink_id_seq', COALESCE(MAX(wwwlink_id), 1) ) FROM public.vs_wwwlink;
SELECT SETVAL('public.vs_zahlweise_zahlweise_id_seq', COALESCE(MAX(zahlweise_id), 1) ) FROM public.vs_zahlweise;


