<?php
/*
 *      vs_chart.php
 *      
 *      Copyright (C)  2005 - 2012 © Dr. Norbert Klumpp
 *      
 *      Copyright (C)  2013 - 2015 © Vetsoft Office Projekt <http://www.http://www.vetsoft.de/>
 *                     Dirk Walter (DW),
 *                     Klaus Munsteiner (KM), 
 *                     Markus Angeli (MA)
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */




//Rückentwicklung der vs_2 library gleichen Namens

//2014-09-04 KM: Fuer PHP5 "var" durch "public" ersetzt.

//include_once "vs_lib_inc.lib.php";

/**
* Typen :
* 1 : Tortengrafik (pie chart)
* 2 : Tortengrafik (pie chart) mit Schatten (3D)
* 3 : Balkengrafik
* 4 : Liniengrafik
*/

class vs_chart 
{
	public $typ;
	public $image;
	public $data;
	public $defs;
	public $img_width;
	public $img_height;
	public $palette;
	//allgemein
	public $id;
	public $title;
	public $width;
	public $height;
	public $background_color; //Hintergrundfarbe der Grafik
	public $grid_width; //Dicke des datengitters
	public $grid_color; //Farbe des Datengitters
	public $grid_info_width; //Breite des Anzeigebereich des vertikalen Datengrids (z.B. Einheiten)
	// Rand Abstaende Grafik
	public $img_margin_x;   //linker und rechter Rand
	public $img_margin_y;   //oberer und unterer Rand
	//3D PieChart
	public $img_shadow_y;  //Schatten unten beim 3D PieChart
	//Balkengrafik
	public $bar_width_autocalc; //1 oder 0. Falls 1:Balkenbreite wird berechnet, angegebene Bildbreite bleibt; sonst: Balkenbreite bleibt, Bildbreite wird angepasst
	public $bar_width; //Breite der Balken
	public $bar_diff; //Abstaende der Balken einer Gruppe (Datensatz)
	public $bar_group_diff;  //Abstande der Balkengruppen(Datensaetze)
	//Liniengrafik
	public $line_real_x; //x_abstände werden im echten Abstand dargestellt (moeglich nur bei numerischen Indizes des $data-arrays)
	public $line_standard_x;  //standard-Abstand der x-Punkte(falls  $line_real_x = 0)
	//Legende
	public $leg_width; //Breite des Bereiches für die Legende
	public $leg_font_file;   //Font fuer Beschriftung, Legende
	public $leg_font_size;  //Größe der Font der Legende
	public $leg_font_angle;   //Schriftrichtung der Legende (0=horizontal)
	public $leg_font_diff_x;   // Versatz der Schrift der Legende nach rechts(bezgl. des entspr. farbigen Rechteckes)
	public $leg_font_diff_y;   // Versatz der Schrift der Legende nach unten(bezgl. des entspr. farbigen Rechteckes)
	public $leg_font_color;  //Schriftfarbe der Legende
	public $leg_rect_width;  //Breite des farbigen Rechteckes der Legende
	public $leg_rect_height;  //Hoehe des farbigen Rechteckes der Legende
	public $leg_rect_diff;  //vertikaler Abstand der Rechtecke der Legende
	//Legende unten (Balkengrafik,Liniendiagramm)
	public $bottom_leg_visible; //ueberhaupt anzeigen? (1/0)
	public $bottom_leg_height; //Hoehe der Legende unten, wenn 0, keine untere Legende anzeigen
	public $bottom_leg_font_size;  //Größe der Font der Legende unten
	public $bottom_leg_font_angle; //Richtung der Schrift der Legende unten (0 ist normal von rechts nach links)
	public $bottom_leg_font_diff_y; //Versatz der Schrift der Legende unten nach unten (Wichtig bei Winkeln <>0)
	public $bottom_leg_font_diff_x; //Versatz der Schrift der Legende unten nach rechts 
	//Legende links (Balkengrafik,Liniendiagramm)
	public $left_leg_visible; //ueberhaupt anzeigen? (1/0)
	public $left_leg_width;  //Breite der Legende unten, wenn 0, keine untere Legende anzeigen
	public $left_leg_font_size;  //Größe der Font der Legende unten
	public $left_leg_font_angle; //Richtung der Schrift der Legende unten (0 ist normal von rechts nach links)
	public $left_leg_font_diff_y; //Versatz der Schrift der Legende unten nach unten (Wichtig bei Winkeln <>0)
	public $left_leg_font_diff_x; //Versatz der Schrift der Legende unten nach rechts 
	
	function __construct($id,$title,$width,$height,$typ,$data,$defs)
	{
		$this->id=$id;
		$this->title=$title;
		$this->width=$width;
		$this->height=$height;
		$this->css_class="vs_chart";
		$this->data=$data;
		$this->defs=$defs;
		$this->img_width=intval($this->width);
		$this->img_height=intval($this->height);
		$this->typ=$typ;
		//allgemein
		$this->background_color="white";
		$this->grid_width=1; 
		$this->grid_color="black";
		$this->grid_info_width=30;
		//Randabstaende
		$this->img_margin_x =3;
		$this->img_margin_y =3;
		//3 D_pie_chart
		$this->img_shadow_y=10;
		//Balkengrafik
		$this->bar_width_autocalc=1;
		$this->bar_width=15;
		$this->bar_diff=0;
		$this->bar_group_diff=20;
		//Liniengrafik
		$this->line_real_x=1;
		$this->line_standard_x=90;
		//Legende
		$this->leg_width =70;
		//2014-01-01 KM: Hier waren der Pfad falsch
		// was soll eine Variable "vs_font_path" in der config.inc.php, wenn alles in vs/vs_lib/font/ liegt?
		$this->leg_font_file="../vs_lib/font/FreeSans.ttf";
		$this->leg_font_size=8;
		$this->leg_font_angle=0;
		$this->leg_font_diff_x=5;
		$this->leg_font_diff_y=12;
		$this->leg_rect_width=25; 
		$this->leg_rect_height=15;
		$this->leg_rect_diff=10; 
		$this->leg_font_color="black";
		//Legende unten (Balken-, Liniendiagramm)
		$this->bottom_leg_visible=1;
		$this->bottom_leg_height=20; 
		$this->bottom_leg_font_size=8;
		$this->bottom_leg_font_angle=0; 
		$this->bottom_leg_font_diff_y=3; 
		$this->bottom_leg_font_diff_x=0; 
		//Legende links am Gitter(Balken-, Liniendiagramm)
		$this->left_leg_visible=1;
		$this->left_leg_width=20; 
		$this->left_leg_font_size=8;
		$this->left_leg_font_angle=0; 
		$this->left_leg_font_diff_y=3; 
		$this->left_leg_font_diff_x=0; 
		//Palette ggf. aus dem Objekt auslagern
		$this->palette=array(
        'aliceblue'     => array(240, 248, 255),
        'antiquewhite'  => array(250, 235, 215),
        'aqua'          => array(0, 255, 255),
        'aquamarine'    => array(127, 255, 212),
        'azure'         => array(240, 255,  255),
        'beige'         => array(245, 245, 220),
        'bisque'        => array(255, 228, 196),
        'black'         => array(0, 0, 0),
        'blanchedalmond'=> array(255, 235, 205),
        'blue'          => array(0, 0, 255),
        'blueviolet'    => array(138, 43, 226),
        'brown'         => array(165, 42, 42),
        'burlywood'     => array(222, 184, 135),
        'cadetblue'     => array(95, 158, 160),
        'chartreuse'    => array(127, 255, 0),
        'chocolate'     => array(210, 105, 30),
        'coral'         => array(255, 127, 80),
        'cornflowerblue'=> array(100, 149,  237),
        'cornsilk'      => array(255, 248, 220),
        'crimson'       => array(220, 20, 60),
        'cyan'          => array(0, 255, 255),
        'darkblue'      => array(0, 0, 13),
        'darkcyan'      => array(0, 139, 139),
        'darkgoldenrod' => array(184, 134, 11),
        'darkgray'      => array(169, 169, 169),
        'darkgreen'     => array(0, 100, 0),
        'darkkhaki'     => array(189, 183, 107),
        'darkmagenta'   => array(139, 0, 139),
        'darkolivegreen'=> array(85, 107, 47),
        'darkorange'    => array(255, 140, 0),
        'darkorchid'    => array(153, 50, 204),
        'darkred'       => array(139, 0, 0),
        'darksalmon'    => array(233, 150, 122),
        'darkseagreen'  => array(143, 188, 143),
        'darkslateblue' => array(72, 61, 139),
        'darkslategray' => array(47, 79, 79),
        'darkturquoise' => array(0, 206, 209),
        'darkviolet'    => array(148, 0, 211),
        'deeppink'      => array(255, 20, 147),
        'deepskyblue'   => array(0, 191, 255),
        'dimgray'       => array(105, 105, 105),
        'dodgerblue'    => array(30, 144, 255),
        'firebrick'     => array(178, 34, 34),
        'floralwhite'   => array(255, 250, 240),
        'forestgreen'   => array(34, 139, 34),
        'fuchsia'       => array(255, 0, 255),
        'gainsboro'     => array(220, 220, 220),
        'ghostwhite'    => array(248, 248, 255),
        'gold'          => array(255, 215, 0),
        'goldenrod'     => array(218, 165, 32),
        'gray'          => array(128, 128, 128),
        'green'         => array(0, 128, 0),
        'greenyellow'   => array(173, 255, 47),
        'honeydew'      => array(240, 255, 240),
        'hotpink'       => array(255, 105, 180),
        'indianred'     => array(205, 92, 92),
        'indigo'        => array(75, 0, 130),
        'ivory'         => array(255, 255, 240),
        'khaki'         => array(240, 230, 140),
        'lavender'      => array(230, 230, 250),
        'lavenderblush' => array(255, 240, 245),
        'lawngreen'     => array(124, 252,  0),
        'lemonchiffon'  => array(255, 250, 205),
        'lightblue'     => array(173, 216, 230),
        'lightcoral'    => array(240, 128, 128),
        'lightcyan'     => array(224, 255, 255),
        'lightgoldenrodyellow' => array(250, 250, 210),
        'lightgreen'    => array(144, 238, 144),
        'lightgrey'     => array(211, 211, 211),
        'lightpink'     => array(255, 182, 193),
        'lightsalmon'   => array(255, 160, 122),
        'lightseagreen' => array(32, 178, 170),
        'lightskyblue'  => array(135, 206, 250),
        'lightslategray'=> array(119, 136, 153),
        'lightsteelblue'=> array(176, 196, 222),
        'lightyellow'   => array(255, 255, 224),
        'lime'          => array(0, 255, 0),
        'limegreen'     => array(50, 205, 50),
        'linen'         => array(250, 240, 230),
        'magenta'       => array(255, 0, 255),
        'maroon'        => array(128, 0, 0),
        'mediumaquamarine' => array(102, 205, 170),
        'mediumblue'    => array(0, 0, 205),
        'mediumorchid'  => array(186, 85, 211),
        'mediumpurple'  => array(147, 112, 219),
        'mediumseagreen'=> array(60, 179, 113),
        'mediumslateblue'   => array(123, 104, 238),
        'mediumspringgreen' => array(0, 250, 154),
        'mediumturquoise'   => array(72, 209, 204),
        'mediumvioletred'   => array(199, 21, 133),
        'midnightblue'  => array(25, 25, 112),
        'mintcream'     => array(245, 255, 250),
        'mistyrose'     => array(255, 228, 225),
        'moccasin'      => array(255, 228, 181),
        'navajowhite'   => array(255, 222, 173),
        'navy'          => array(0, 0, 128),
        'oldlace'       => array(253, 245, 230),
        'olive'         => array(128, 128, 0),
        'olivedrab'     => array(107, 142, 35),
        'orange'        => array(255, 165, 0),
        'orangered'     => array(255,69,0),
        'orchid'        => array(218,112,214),
        'palegoldenrod' => array(238,232,170),
        'palegreen'     => array(152,251,152),
        'paleturquoise' => array(175,238,238),
        'palevioletred' => array(219,112,147),
        'papayawhip'    => array(255,239,213),
        'peachpuff'     => array(255,218,185),
        'peru' => array(205,133,63),
        'pink' => array(255,192,203),
        'plum' => array(221,160,221),
        'powderblue' => array(176,224,230),
        'purple' => array(128,0,128),
        'red' => array(255,0,0),
        'rosybrown' => array(188,143,143),
        'royalblue' => array(65,105,225),
        'saddlebrown' => array(139,69,19),
        'salmon' => array(250,128,114),
        'sandybrown' => array(244,164,96),
        'seagreen' => array(46,139,87),
        'seashell' => array(255,245,238),
        'sienna' => array(160,82,45),
        'silver' => array(192,192,192),
        'skyblue' => array(135,206,235),
        'slateblue' => array(106,90,205),
        'slategray' => array(112,128,144),
        'snow' => array(255,250,250),
        'springgreen' => array(0,255,127),
        'steelblue' => array(70,130,180),
        'tan' => array(210,180,140),
        'teal' => array(0,128,128),
        'thistle' => array(216,191,216),
        'tomato' => array(255,99,71),
        'turquoise' => array(64,224,208),
        'violet' => array(238,130,238),
        'wheat' => array(245,222,179),
        'white' => array(255,255,255),
        'whitesmoke' => array(245,245,245),
        'yellow' => array(255,255,0),
        'yellowgreen' => array(154,205,50)
		);
		
	}
	
	//ermittelt den hoechsten Wert der Datenmenge
	private function max_data()
	{
		$max=0;
		reset($this->data);
		while (list($key,$value)=each($this->data))
		{
			$m=max($value);
			if($m>$max)
			$max=$m;
		}	
		return $max;
	}	
	
	private function legend_and_colors($defs,&$icolor,&$scolor)
	{
		$z=0;  //Zähler, für die y-Positionen der Legende
		//Schriftfarbe
		$font_color=$this->palette[$this->leg_font_color];
		$fontcolor=imagecolorallocate($this->image,$font_color[0],$font_color[1],$font_color[2]);
		//Grafikfarben
		while(list($key,$val)= each($defs))
		{
			//Farben erzeugen
			$frag_color=$this->palette[$val];
			$icolor[$key]=imagecolorallocate($this->image,$frag_color[0],$frag_color[1],$frag_color[2]);
			//Schattenfarben
			for ($t=0;$t<3;$t++)
			{
				if($frag_color[$t] >0)
				$frag_color[$t]=$frag_color[$t] -40;
				if ($frag_color[$t] <0) $frag_color[$t] =0;
			}	
			$scolor[$key]=imagecolorallocate($this->image,$frag_color[0],$frag_color[1],$frag_color[2]);
			//Legende - farb. Rechteck
			imagefilledrectangle($this->image,$this->img_margin_x,$this->img_margin_y+($z*($this->leg_rect_height+$this->leg_rect_diff)) ,$this->img_margin_x+$this->leg_rect_width, $this->img_margin_y+($z*($this->leg_rect_height+$this->leg_rect_diff))+$this->leg_rect_height, $icolor[$key]);
			//Legende - zug. Text
			imagettftext($this->image,$this->leg_font_size, $this->leg_font_angle, $this->img_margin_x+$this->leg_rect_width+$this->leg_font_diff_x, $this->img_margin_y+($z*($this->leg_rect_height+$this->leg_rect_diff))+$this->leg_font_diff_y, $fontcolor, $this->leg_font_file,$key);
			$z++;
		}
	}	
	
	private function draw_pie_chart($data,$defs)
	{
		//Bild erzeugen
		$this->image= imagecreate($this->img_width,$this->img_height);
		//Hintergrundfarbe erzeugen
		$background_color=$this->palette[$this->background_color];
		$backgroundcolor=imagecolorallocate($this->image,$background_color[0],$background_color[1],$background_color[2]);
		$black=imagecolorallocate($this->image,0,0,0);
		//x-Werte der Ellipsenmittelpunktes
		$mid_x=(intval($this->img_width)-$this->leg_width)/2 + $this->leg_width ; 
		//y-Werte des Ellipsenmittelpunktes
		$mid_y=intval($this->img_height/2) ;	
		//Breite der Ellipse
		$pie_x=intval($this->img_width) - $this->leg_width - $this->img_margin_x*2;   
		//Hoehe der Ellipse
		$pie_y=$mid_y*2 - $this->img_margin_y*2;															   
		//Definition auswerten, Farben und Legende zur Verfuegung stellen
		$this->legend_and_colors($defs,$icolor,$scolor);
		//Daten auswerten
		$start=0;
		while(list($key,$val)= each($data))
		{
			$sum=array_sum($data);
			$wert=$val/$sum*360;
			imagefilledarc($this->image,$mid_x,$mid_y,$pie_x,$pie_y,$start,($wert+$start),$icolor[$key],IMG_ARC_PIE);
			$start=$start+$wert;
		}	
		imagepng($this->image,$this->id."_chart.png");
		imagedestroy($this->image);
	}	
		
		
	private function draw_3D_pie_chart($data,$defs)
	{
		//Bild erzeugen
		$this->image= imagecreate($this->img_width,$this->img_height);
		//Hintergrundfarbe erzeugen
		$background_color=$this->palette[$this->background_color];
		$backgroundcolor=imagecolorallocate($this->image,$background_color[0],$background_color[1],$background_color[2]);
		$black=imagecolorallocate($this->image,0,0,0);
		//x-Werte der Ellipsenmittelpunktes
		$mid_x=(intval($this->img_width)-$this->leg_width)/2 + $this->leg_width ; 
		//y-Werte des Ellipsenmittelpunktes
		$mid_y=intval($this->img_height/2)- $this->img_shadow_y/2;	
		//Breite der Ellipse
		$pie_x=intval($this->img_width) - $this->leg_width - $this->img_margin_x*2;   
		//Hoehe der Ellipse
		$pie_y=$mid_y*2 - $this->img_margin_y*2-$this->img_shadow_y;															   
		//Definition auswerten, Farben und Legende zur Verfuegung stellen
		$this->legend_and_colors($defs,$icolor,$scolor);
		//Daten 
		//zuerst der Schatten
		for ($i=$mid_y+$this->img_shadow_y;$i>$mid_y;$i--)
		{
			$start=0;
			reset($data);
			while(list($key,$val)= each($data))
			{
				$sum=array_sum($data);
				$wert=$val/$sum*360;
				imagefilledarc($this->image,$mid_x,$i,$pie_x,$pie_y,$start,($wert+$start),$scolor[$key],IMG_ARC_PIE);
				$start=$start+$wert;
			}
		}
		//dann die eigentliche Ellipse	
		$start=0;
		reset($data);
		while(list($key,$val)= each($data))
		{
			$sum=array_sum($data);
			$wert=$val/$sum*360;
			imagefilledarc($this->image,$mid_x,$mid_y,$pie_x,$pie_y,$start,($wert+$start),$icolor[$key],IMG_ARC_PIE);
			$start=$start+$wert;
		}	
		imagepng($this->image,$this->id."_chart.png");
		imagedestroy($this->image);
	}		
		
	private function draw_bar_chart($data,$defs)
	{
		if ($this->bar_width_autocalc==1)
		{
			//die Balkenbreite und Gruppenabstaende der Bildbreite anpassen (Gruppenabstand wird der Einfachkeit halber gleich der Balkenbreite berechnet) 
			//Breite Kurve
			$imgwidth=$this->img_width-($this->leg_width +$this->grid_info_width+$this->img_margin_x*2);
			//Zahl der Balken+Gruppenabstaende
			$barcount=count($data)*count($defs)+count($data)-1;
			//Balkenbreite
			$this->bar_width = floor($imgwidth/$barcount);  //abrunden
			$this->bar_group_diff = ceil($imgwidth/$barcount); //aufrunden
		}
		else
		{
			// Breite des Bildes berechnen (Legende + Zahl der datensaetze * ((Zahl derBalken/Datensatz *Breite+Abstand )+Abstand der Datensaetze)
			$this->img_width=$this->leg_width +$this->grid_info_width+ count($data)* ((count($defs)*($this->bar_width+$this->bar_diff)+$this->bar_group_diff))+$this->img_margin_x*2 - $this->bar_group_diff ; //letzte Gruppe hat kein diff mehr
		}	
		//Bild erzeugen
		$this->image= imagecreate($this->img_width,$this->img_height);
		//Hintergrundfarbe erzeugen
		$background_color=$this->palette[$this->background_color];
		$backgroundcolor=imagecolorallocate($this->image,$background_color[0],$background_color[1],$background_color[2]);
		//Definition auswerten, Farben und Legende zur Verfuegung stellen
		$this->legend_and_colors($defs,$icolor,$scolor);
		//Wertegitter, Farbe erzeugen
		$grid_color=$this->palette[$this->grid_color];
		$gridcolor=imagecolorallocate($this->image,$grid_color[0],$grid_color[1],$grid_color[2]);
		$baseline=$this->img_height-$this->img_margin_y-$this->bottom_leg_height;  //Grundline der Daten; die horizonzale Linie liegt auf Baseline+1
		$maxdata=$this->max_data();
		//senkrechte Linie (links)
		imageline  ($this->image ,$this->img_margin_x+$this->leg_width+$this->grid_info_width,$this->img_margin_y ,$this->img_margin_x+$this->leg_width+$this->grid_info_width,$baseline+1,$gridcolor);
		//Beschriftung der senkrechten Linie 
		//unten (0)
		imagettftext($this->image,$this->left_leg_font_size, $this->left_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width-$this->left_leg_width, $baseline, $gridcolor, $this->leg_font_file,"0");
		//mitte
		imagettftext($this->image,$this->left_leg_font_size, $this->left_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width-$this->left_leg_width, round(($baseline-$this->img_margin_y)/2)+$this->left_leg_font_size, $gridcolor, $this->leg_font_file,round($maxdata/2));
		//oben,Maximalwert
		imagettftext($this->image,$this->left_leg_font_size, $this->left_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width-$this->left_leg_width, $this->img_margin_y+$this->left_leg_font_size, $gridcolor, $this->leg_font_file,$maxdata);
		//horizontale Linie (unten)
		imageline  ($this->image ,$this->img_margin_x+$this->leg_width+$this->grid_info_width,$baseline+1 ,$this->img_width-$this->img_margin_x,$baseline+1,$gridcolor);
		//DATEN
		//aeussere Schleife,Gesamtdaten
		$as_nr=0; //Nummer des Datensatzes
		
		$is_c=count($defs); //Zahl der Felder des  Datensatzes
		while(list($a_key,$a_val)= each($data))
		{
			if ($this->bottom_leg_visible=="1")
			imagettftext($this->image,$this->bottom_leg_font_size, $this->bottom_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width+$as_nr*($is_c *$this->bar_width+$this->bar_group_diff)+$this->bottom_leg_font_diff_x, $baseline + $this->leg_font_size+$this->bottom_leg_font_diff_y  , $gridcolor, $this->leg_font_file,$a_key);
			$is_nr=0;
			while(list($key,$val)=each($a_val))
			{
				$y1=$baseline-round($val/$maxdata*($baseline - $this->img_margin_y));  //oberer y-Wert der Balken                 
				imagefilledrectangle($this->image,1+$this->img_margin_x+$this->leg_width+$this->grid_info_width+$as_nr*($is_c *$this->bar_width+$this->bar_group_diff) +  $is_nr*$this->bar_width, $y1 , $this->img_margin_x+ $this->leg_width+$this->grid_info_width+$this->bar_width+$as_nr*($is_c *$this->bar_width+$this->bar_group_diff) +  $is_nr* $this->bar_width, $baseline,$icolor[$key]);
				$is_nr ++;
			}	
			$as_nr ++;
		}	
		imagepng($this->image,$this->id."_chart.png");
		imagedestroy($this->image);
	}	
			
	private function draw_line_chart($data,$defs)
	{
		//Abstaende der Punkte festlegen
		$x_width=$this->img_width-($this->leg_width +2*$this->grid_info_width+2*$this->img_margin_x); //2 x grid_info_width wegen der Beschriftung unten
		$this->line_standard_x= round($x_width/(count($data)-1));
		//Bild erzeugen
		$this->image= imagecreate($this->img_width,$this->img_height);
		//Hintergrundfarbe erzeugen
		$background_color=$this->palette[$this->background_color];
		$backgroundcolor=imagecolorallocate($this->image,$background_color[0],$background_color[1],$background_color[2]);
		//Definition auswerten, Farben und Legende zur Verfuegung stellen
		$this->legend_and_colors($defs,$icolor,$scolor);
		//Wertegitter, Farbe erzeugen
		$grid_color=$this->palette[$this->grid_color];
		$gridcolor=imagecolorallocate($this->image,$grid_color[0],$grid_color[1],$grid_color[2]);
		$baseline=$this->img_height-$this->img_margin_y-$this->bottom_leg_height;  //Grundline der Daten; die horizonzale Linie liegt auf Baseline+1
		$maxdata=$this->max_data();
		//senkrechte Linie (links)
		imageline  ($this->image ,$this->img_margin_x+$this->leg_width+$this->grid_info_width,$this->img_margin_y ,$this->img_margin_x+$this->leg_width+$this->grid_info_width,$baseline+1,$gridcolor);
		//Beschriftung der senkrechten Linie 
		//unten (0)
		imagettftext($this->image,$this->left_leg_font_size, $this->left_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width-$this->left_leg_width, $baseline, $gridcolor, $this->leg_font_file,"0");
		//mitte
		imagettftext($this->image,$this->left_leg_font_size, $this->left_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width-$this->left_leg_width, round(($baseline-$this->img_margin_y)/2)+$this->left_leg_font_size, $gridcolor, $this->leg_font_file,round($maxdata/2));
		//oben,Maximalwert
		imagettftext($this->image,$this->left_leg_font_size, $this->left_leg_font_angle,  $this->img_margin_x+$this->leg_width+$this->grid_info_width-$this->left_leg_width, $this->img_margin_y+$this->left_leg_font_size, $gridcolor, $this->leg_font_file,$maxdata);
		//horizontale Linie (unten)
		imageline  ($this->image ,$this->img_margin_x+$this->leg_width+$this->grid_info_width,$baseline+1 ,$this->img_width-$this->img_margin_x,$baseline+1,$gridcolor);
		//DATEN
		//diesmal alle Felder hintereinander abarbeiten 
		$bottom_leg_written="n";
		while(list($def_key,$def_val) =each($defs))
		{
			$as_nr=0; //Nummer des Datensatzes
			unset($oldval);
			reset($data);
			while(list($data_key,$data_val) =each($data))
			{
				$val=$data_val[$def_key];
				$py2=$baseline-round($val/$maxdata*($baseline - $this->img_margin_y)); 
				$px2=$this->img_margin_x+$this->leg_width+$this->grid_info_width+$as_nr*$this->line_standard_x;
				if(isset($oldval))
				{
					$py1=$baseline-round($oldval/$maxdata*($baseline - $this->img_margin_y)); 
					$px1=$this->img_margin_x+$this->leg_width+$this->grid_info_width+($as_nr-1)*$this->line_standard_x;
					//Linie ziehen
					imageline  ($this->image ,$px1,$py1,$px2,$py2,$icolor[$def_key]);
					//imageline  ($this->image ,$px1,$py1+1,$px2,$py2+1,$icolor[$def_key]); //waere dickere Linie
				}
				//vorherigen Wert aktualisieren
				$oldval=$val;
				//Knickpunkte markieren
				imagefilledrectangle($this->image,$px2-2,$py2-2, $px2+2,$py2+2,$icolor[$def_key]);
				//Legende unten
				if ($bottom_leg_written==="n")
				{
					imagettftext($this->image,$this->bottom_leg_font_size, $this->bottom_leg_font_angle,$px2,$baseline+ $this->leg_font_size+$this->bottom_leg_font_diff_y, $gridcolor, $this->leg_font_file,$data_key);
					
				}
				$as_nr++;
			}	
			$bottom_leg_written="y";
		}	
		imagepng($this->image,$this->id."_chart.png");
		imagedestroy($this->image);
	}	
				
		
	function write()
	{
		switch ($this->typ)
		{
			case "1":  //pie chart
				$this->draw_pie_chart($this->data[0],$this->defs);
			break;
			case "2":
				$this->draw_3D_pie_chart($this->data[0],$this->defs);
			break;
			case "3":
				$this->draw_bar_chart($this->data,$this->defs);
			break;
			case "4":
				$this->draw_line_chart($this->data,$this->defs);
			break;
		}
		//Ausgabe
		echo "<div style= \"width:$this->width px; height:$this->height px; \"><img src=\"".$this->id. "_chart.png\" alt=\"$this->title\" ></div>";
		
	}

}


?>
